// C++ informative line for the emacs editor: -*- C++ -*-
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 * Copyright by The HDF Group.                                               *
 * Copyright by the Board of Trustees of the University of Illinois.         *
 * All rights reserved.                                                      *
 *                                                                           *
 * This file is part of HDF5.  The full HDF5 copyright notice, including     *
 * terms governing use, modification, and redistribution, is contained in    *
 * the files COPYING and Copyright.html.  COPYING can be found at the root   *
 * of the source code distribution tree; Copyright.html can be found at the  *
 * root level of an installed copy of the electronic HDF5 document set and   *
 * is linked from the top-level documents page.  It can also be found at     *
 * http://hdfgroup.org/HDF5/doc/Copyright.html.  If you do not have          *
 * access to either file, you may request a copy from help@hdfgroup.org.     *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef __H5Classes_H
#define __H5Classes_H

namespace H5 {
	class Exception;
	class IdComponent;
	class H5Location;
	class H5Object;
	class PropList;
	class FileCreatPropList;
	class FileAccPropList;
	class DSetCreatPropList;
	class DSetMemXferPropList;
	class DTypePropList;
	class DataType;
	class DataSpace;
	class AtomType;
	class PredType;
	class EnumType;
	class IntType;
	class FloatType;
	class StrType;
	class CompType;
	//class RefType;
	class AbstractDs;
	class DataSet;
	class Group;
	class H5File;
	class Attribute;
	class H5Library;
}
#endif // __H5Classes_H
