



where gif2h5
if errorlevel 1 exit 1
where h52gif
if errorlevel 1 exit 1
where h5copy
if errorlevel 1 exit 1
where h5debug
if errorlevel 1 exit 1
where h5diff
if errorlevel 1 exit 1
where h5dump
if errorlevel 1 exit 1
where h5import
if errorlevel 1 exit 1
where h5jam
if errorlevel 1 exit 1
where h5ls
if errorlevel 1 exit 1
where h5mkgrp
if errorlevel 1 exit 1
where h5repack
if errorlevel 1 exit 1
where h5repart
if errorlevel 1 exit 1
where h5stat
if errorlevel 1 exit 1
where h5unjam
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\lib\\hdf5.lib exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\bin\\hdf5.dll exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\lib\\hdf5_cpp.lib exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\bin\\hdf5_cpp.dll exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\lib\\hdf5_hl.lib exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\bin\\hdf5_hl.dll exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\lib\\hdf5_hl_cpp.lib exit 1
if errorlevel 1 exit 1
if not exist %PREFIX%\\Library\\bin\\hdf5_hl_cpp.dll exit 1
if errorlevel 1 exit 1
exit 0
